@@ -1,12 +1,10 @@
 # Palm::Datebook.pm
 # 
-# Perl class for dealing with Palm DateBook databases. 
+# Perl class for dealing with Palm DateBook and Calendar databases. 
 #
 #	Copyright (C) 1999-2001, Andrew Arensburger.
 #	You may distribute this file under the terms of the Artistic
 #	License, as specified in the README file.
-#
-# $Id: Datebook.pm,v 1.20 2003/10/10 23:19:58 azummo Exp $
 
 use strict;
 package Palm::Datebook;
@@ -16,14 +14,14 @@ use Palm::StdAppInfo();
 use vars qw( $VERSION @ISA );
 
 # One liner, to allow MakeMaker to work.
-$VERSION = do { my @r = (q$Revision: 1.20 $ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r };
+$VERSION = '1.012';
 
 @ISA = qw( Palm::StdAppInfo Palm::Raw );
 
 
 =head1 NAME
 
-Palm::Datebook - Handler for Palm DateBook databases.
+Palm::Datebook - Handler for Palm DateBook and Calendar databases.
 
 =head1 SYNOPSIS
 
@@ -32,7 +30,7 @@ Palm::Datebook - Handler for Palm DateBook databases.
 =head1 DESCRIPTION
 
 The Datebook PDB handler is a helper class for the Palm::PDB package.
-It parses DateBook databases.
+It parses DateBook and Calendar databases.
 
 =head2 AppInfo block
 
@@ -54,8 +52,8 @@ This is a scalar, the raw data of the sort block.
     $record->{year}
 
 The day, month and year of the event. The day and month start at 1
-(I<i.e.>, for January, C<$record-E<gt>{month}> is set to 1). The year
-is a four-digit number (for dates in 2001, C<$record-E<gt>{year}> is
+(I<i.e.>, for January, C<< $record->{month} >> is set to 1). The year
+is a four-digit number (for dates in 2001, C<< $record->{year} >> is
 "2001").
 
 For repeating events, these fields specify the first date at which the
@@ -86,13 +84,13 @@ I<e.g.>, if "unit" is 1 and "advance" is 5, then the alarm will sound
 
 If C<advance> is -1, then there is no alarm associated with this event.
 New records created via C<new_Record> have a 10 minute alarm set by
-default. C<undef $record->{alarm}> to remove this alarm before writing.
+default. C<< undef $record->{alarm} >> to remove this alarm before writing.
 An alarm icon will still show up in the Datebook if the C<alarm>
 field exists, even with C<advance> set to -1.
 
     %{$record->{repeat}}
 
-This has exists iff this is a repeating event.
+This exists iff this is a repeating event.
 
     $record->{repeat}{type}
 
@@ -124,8 +122,8 @@ Monday, and so forth.
 
     $record->{repeat}{start_of_week}
 
-I'm not sure what this is, but the Datebook app appears to perform
-some hairy calculations involving this.
+Day the week starts on (0 for Sunday, 1 for Monday). This affects
+the phase of events that repeat every 2nd (or more) Sunday.
 
 =item 3
 
@@ -189,10 +187,62 @@ three elements: the day, month, and year of the exception.
 
 A text string, the description of the event.
 
+    $record->{location}
+
+A text string, the location (if any) of the event (Calendar database only).
+
     $record->{note}
 
 A text string, the note (if any) attached to the event.
 
+    %{$record->{timezone}}
+
+This exists iff a time zone has been set for the event.
+
+    $record->{timezone}{name}
+
+The time zone name, I<e.g.>, "London" or "San Francisco".
+
+    $record->{timezone}{country}
+
+The country the time zone is in. This is an integer defined in Core/System/PalmLocale.h.
+
+    $record->{timezone}{offset}
+
+This gives the offset from UTC, in minutes, of the time zone.
+
+    $record->{timezone}{dst_adjustment}
+
+This gives the additional offset while daylight savings time is in effect.
+The offset from UTC is $record->{timezone}{offset} + $record->{timezone}{dst_adjustment}
+(actually only 0 or 60 are used).
+
+    $record->{timezone}{custom}
+
+Should indicate whether this location was created by the user, though this
+always seems to be true.
+
+    $record->{timezone}{flags}
+
+Reserved flags.
+
+    $record->{timezone}{start_hour}
+    $record->{timezone}{start_daynum}
+    $record->{timezone}{start_weeknum}
+    $record->{timezone}{start_month}
+    $record->{timezone}{end_hour}
+    $record->{timezone}{end_daynum}
+    $record->{timezone}{end_weeknum}
+    $record->{timezone}{end_month}
+
+These define the period during which daylight savings time is in effect
+if $record->{timezone}{dst_adjustment} is non-zero.
+daynum specifies the day of week (0=Sunday, 6=Saturday) and weeknum specifies the week of month
+(0=1st, 3=4th, 4=last), analagously to the "monthly by day" repeating event.
+I<e.g.>, The "London" time zone has DST starting on the last Sunday of March, which
+is indicated with start_daynum=0 (Sunday), start_weeknum=4 (last week of the month),
+and start_month=3 (March).
+
 =head1 METHODS
 
 =cut
@@ -202,15 +252,18 @@ sub import
 {
 	&Palm::PDB::RegisterPDBHandlers(__PACKAGE__,
 		[ "date", "DATA" ],
+		[ "PDat", "DATA" ],
 		);
 }
 
 =head2 new
 
   $pdb = new Palm::Datebook;
+  $pdb = new Palm::Datebook({app => 'Calendar'});
 
 Create a new PDB, initialized with the various Palm::Datebook fields
-and an empty record list.
+and an empty record list. Creates a Datebook PDB by default.
+Specify the app parameter as 'Calendar' to create a Calendar PDB.
 
 =cut
 #'
@@ -220,13 +273,19 @@ and an empty record list.
 sub new
 {
 	my $classname	= shift;
+	my $params    = $_[0] || {};
 	my $self	= $classname->SUPER::new(@_);
 			# Create a generic PDB. No need to rebless it,
 			# though.
 
-	$self->{name} = "DatebookDB";	# Default
-	$self->{creator} = "date";
-	$self->{type} = "DATA";
+	if ($params->{app} eq 'Calendar' || $self->{creator} eq 'PDat') {
+		$self->{name} ||= "CalendarDB-PDat";	# Default
+		$self->{creator} = "PDat" if $self->{creator} eq "\0\0\0\0";
+	} else {
+		$self->{name} ||= "DatebookDB";	# Default
+		$self->{creator} = "date" if $self->{creator} eq "\0\0\0\0";
+	}
+	$self->{type} = "DATA" if $self->{type} eq "\0\0\0\0";
 	$self->{attributes}{resource} = 0;
 				# The PDB is not a resource database by
 				# default, but it's worth emphasizing,
@@ -250,7 +309,7 @@ sub new
 Creates a new Datebook record, with blank values for all of the fields.
 
 C<new_Record> does B<not> add the new record to C<$pdb>. For that,
-you want C<$pdb-E<gt>append_Record>.
+you want C<< $pdb->append_Record >>.
 
 =cut
 
@@ -281,6 +340,7 @@ sub new_Record
 
 	$retval->{description} = "";
 	$retval->{note} = undef;
+	$retval->{location} = undef;
 
 	return $retval;
 }
@@ -336,6 +396,7 @@ sub ParseRecord
 	my $self = shift;
 	my %record = @_;
 	my $data;
+	my $iscal = ($self->{creator} eq 'PDat');
 
 	delete $record{offset};	# This is useless
 
@@ -377,14 +438,15 @@ sub ParseRecord
 	$record{year} = $year;
 
 	# Flags
-	my $when_changed	= ($flags & 0x8000 ? 1 : 0);
-	my $have_alarm		= ($flags & 0x4000 ? 1 : 0);
-	my $have_repeat		= ($flags & 0x2000 ? 1 : 0);
-	my $have_note		= ($flags & 0x1000 ? 1 : 0);
-	my $have_exceptions	= ($flags & 0x0800 ? 1 : 0);
+	my $when_changed	    = ($flags & 0x8000 ? 1 : 0);
+	my $have_alarm		    = ($flags & 0x4000 ? 1 : 0);
+	my $have_repeat		    = ($flags & 0x2000 ? 1 : 0);
+	my $have_note		    = ($flags & 0x1000 ? 1 : 0);
+	my $have_exceptions	    = ($flags & 0x0800 ? 1 : 0);
 	my $have_description	= ($flags & 0x0400 ? 1 : 0);
+	my $have_location	    = (($iscal && ($flags & 0x0200)) ? 1 : 0);
 
-	$record{other_flags} = $flags & 0x03ff;
+	$record{other_flags} = $flags & ($iscal ? 0x01ff : 0x03ff);
 
 	if ($when_changed)
 	{
@@ -499,7 +561,7 @@ sub ParseRecord
 		}
 	}
 
-	my @fields = split /\0/, $data;
+	my @fields = split /\0/, $data, -1;
 
 	if ($have_description)
 	{
@@ -517,6 +579,37 @@ sub ParseRecord
 		$record{note} = $note;
 	}
 
+	if ($have_location)
+	{
+		my $location;
+
+		$location = shift @fields;
+		$record{location} = $location;
+	}
+
+	my $other_data= join ("\0", @fields);
+
+	if ($iscal && length ($other_data) >= 21 && substr ($other_data, 0, 4) eq 'Bd00') {
+		my $len= unpack ('n', substr ($other_data, 4, 2));
+		if ($len+6 <= length ($other_data)) {
+			my $tzdata= substr ($other_data, 6, $len);
+			$other_data= substr ($other_data, $len+6);
+			@{$record{timezone}}{qw(offset start_hour start_daynum start_weeknum start_month
+											 end_hour   end_daynum   end_weeknum   end_month
+									dst_adjustment country flags name)}= unpack ('n C8 n C2 a*', $tzdata);
+			$record{timezone}{name} =~ s/\0$//;
+			$record{timezone}{offset}=         $record{timezone}{offset}        -65536
+				if $record{timezone}{offset}         > 32767;  # signed short
+			$record{timezone}{dst_adjustment}= $record{timezone}{dst_adjustment}-65536
+				if $record{timezone}{dst_adjustment} > 32767;  # signed short
+			$record{timezone}{custom}= ($record{timezone}{flags} & 0x80) ? 1 : 0;
+			$record{timezone}{flags} &= 0x7f;
+			$record{timezone}{data}= $tzdata;
+		}
+	}
+
+	$record{other_data}= $other_data if $other_data ne '';
+
 	delete $record{data};
 
 	return \%record;
@@ -530,6 +623,7 @@ sub PackRecord
 
 	my $rawDate;
 	my $flags;
+	my $iscal = ($self->{creator} eq 'PDat');
 
 	$rawDate = ($record->{day}            & 0x001f) |
 		  (($record->{month}          & 0x000f) << 5) |
@@ -543,6 +637,7 @@ sub PackRecord
 #  	$flags |= 0x1000 if $record->{note} ne "";
 #  	$flags |= 0x0800 if $#{$record->{exceptions}} >= 0;
 #  	$flags |= 0x0400 if $record->{description} ne "";
+#  	$flags |= 0x0200 if $iscal && $record->{location} ne "";
 
 #  	$retval = pack "C C C C n n",
 #  		$record->{start_hour},
@@ -665,6 +760,14 @@ sub PackRecord
 		$note = $record->{note} . "\0";
 	}
 
+	my $location = undef;
+
+	if ($iscal && defined($record->{location}) && ($record->{location} ne ""))
+	{
+		$flags |= 0x0200;
+		$location = $record->{location} . "\0";
+	}
+
 	$retval = pack "C C C C n n",
 		$record->{start_hour},
 		$record->{start_minute},
@@ -673,11 +776,27 @@ sub PackRecord
 		$rawDate,
 		$flags;
 
-	$retval .= $alarm	if defined($alarm);
-	$retval .= $repeat	if defined($repeat);
+	$retval .= $alarm	    if defined($alarm);
+	$retval .= $repeat	    if defined($repeat);
 	$retval .= $exceptions	if defined($exceptions);
 	$retval .= $description	if defined($description);
-	$retval .= $note	if defined($note);
+	$retval .= $note        if defined($note);
+	$retval .= $location	if defined($location);
+
+	if ($iscal && $record->{timezone}) {
+		my $tzflags= $record->{timezone}{flags};
+		$tzflags |= 0x80 if $record->{timezone}{custom};
+		my $tzdata = pack ('n C8 n C2',
+						   @{$record->{timezone}}{qw(offset start_hour start_daynum start_weeknum start_month
+															  end_hour   end_daynum   end_weeknum   end_month
+													 dst_adjustment country)}, $tzflags);
+		$tzdata .= "$record->{timezone}{name}\0";
+		$retval .= 'Bd00';
+		$retval .= pack ('n', length ($tzdata));
+		$retval .= $tzdata;
+	}
+
+	$retval .= $record->{other_data} if exists $record->{other_data};
 
 	return $retval;
 }
@@ -685,9 +804,17 @@ sub PackRecord
 1;
 __END__
 
+=head1 SOURCE CONTROL
+
+The source is in Github:
+
+	http://github.com/briandfoy/p5-Palm/tree/master
+	
 =head1 AUTHOR
 
-Andrew Arensburger E<lt>arensb@ooblick.comE<gt>
+Alessandro Zummo, C<< <a.zummo@towertech.it> >>
+
+Currently maintained by brian d foy, C<< <bdfoy@cpan.org> >>
 
 =head1 SEE ALSO
 
